// ===================== MSP430F1xxx UART0_SMCLK  α׷ =================
//  CPU : MSP430F1611
//     ¥ : 2008-
//      : Ver1.0
//    : www.cpuplaza.com
// Ϸ : mspgcc+Dev-C++
// [ ]  : ACLK : 32.768Khz
//            MCLK : DCO(3.6864MHz)
//            SMCLK: DCO(3.6864MHz)
// ----------------------------------------------------------------------------
// ⺻ 
#include <io.h>
#include <stdio.h>
#include <signal.h>
#include <iomacros.h>

//  
#define		DELTA_1	900					// DCO = DELATA_1 * 4096 = 3.6864MHz
#define		DELTA_2	250					// DCO = DELATA_2 * 4096 = 1.024000MHz
#define		DELTA_3	64					// DCO = DELATA_3 * 4096 = 0.262144MHz

#define		DELTA	DELTA_1					// DCO = 3.6864MHz

// Լ 
void cpu_setup(void);						// CPU ʱȭ
void Set_DCO(void);						// DCO 

void com_set(long baud);					//  
int putchar(int c);						// printf() Լ ϱ  Լ
void putchar_0(unsigned char datac);				//  1Ʈ ۽
void puts_0(unsigned char dbuf[]);				//  ڿ 

//  
volatile unsigned char com0_rx_data;				// UART0  
volatile int com0_rx_eflg = 0;					// UART0  ͷƮ Ϸ ÷

// ===============================  α׷ ==============================
// [μ]: void
// [] UART SMCLK(3.6864MHz) 
// ---------------------------------------------------------------------------
int main(void)
{
int datai = 1345;
	
	cpu_setup();						// cpuʱȭ
	com_set(9600);						//  
	_EINT();						// Enable interrupts

	puts_0("serial test program\r\n");			// string tx
	printf("%s : %d\r\n","test printf",datai);
	while(1){
		if(com0_rx_eflg){				// UART1  Ϸ
			putchar_0(com0_rx_data);		// ۽
			com0_rx_eflg = 0;			// ͷƮ ÷ Clear
		}
	}
}

// ===================================== CPUʱȭ ============================
// [μ] void
// ----------------------------------------------------------------------------
void cpu_setup(void)
{
unsigned dly;

	WDTCTL = WDTPW + WDTHOLD;				// Init watchdog timer(stop)
	
// XTX  : Low Freqency
// DIVx : ACLK(4.096Khz) = LXFTCLK(32.768KHz) / 8
	BCSCTL1 = XT2OFF | DIVA1 | DIVA0;
	for(dly = 0; dly < 0x4000U; dly++) ;			// XTAL  ð 
	Set_DCO();						// DCO 	

// DIVx : ACLK = LXFTCLK(32.768KHz) / 1
	BCSCTL1 &= ~(DIVA1 | DIVA0);
}

// =================================== DCO  ===============================
// [μ] void
// ----------------------------------------------------------------------------
void Set_DCO(void)
{
unsigned int Compare, Oldcapture = 0;

// CMx  : Capture on rising edge(01)
// CCISx: CCIxB(01)
// CAP  : Capture Mode(1)
	CCTL2 = CCIS0 + CM0 + CAP;				// Define CCR2, CAP, ACLK

// TASSELx : Timer Clock Source ACLK(01)
// TACLR : TimerA Clear
// MCx   : Continuos mode : The timer counts up to 0xffffh(10)
	TACTL = TASSEL1 + TACLR + MC1;				// SMCLK, continous mode
	while(1){
		while (!(CCIFG & CCTL2));			// Wait until capture occured
		CCTL2 &= ~CCIFG;				// Capture occured, clear flag
		Compare = CCR2;					// Get current captured SMCLK
		Compare = Compare - Oldcapture;			// SMCLK difference
		Oldcapture = CCR2;				// Save current captured SMCLK
		if(DELTA == Compare){
			break;					// if equal, leave "while(1)"
		}
		else if(DELTA < Compare){			// DCO is too fast, slow it down
			DCOCTL--;
			if (DCOCTL == 0xFF){
				if (!(BCSCTL1 == (XT2OFF + DIVA_3)))
				BCSCTL1--;			// Did DCO roll under?, Sel lower RSEL
			}
		}
		else{
			DCOCTL++;
			if(DCOCTL == 0x00){
				if (!(BCSCTL1 == (XT2OFF + DIVA_3 + 0x07)))
				BCSCTL1++;			// Did DCO roll over? Sel higher RSEL
			}
		}
	}
	CCTL2 = 0;						// Stop CCR2
	TACTL = 0;						// Stop Timer_A
}

// ================================   =================================
// [μ] long baud
// Item1: Ʈ (300 - 115200Bps)
// [] SMCLK(3.6864MHz)
// ----------------------------------------------------------------------------
void com_set(long baud)
{
unsigned datau;

// UART Mode,8Bit Data, NON Parity, 1 STOP
	UCTL0 = CHAR;
	UTCTL0 = SSEL1;						// Source Clock UCLK = SMCLK
	datau = (unsigned)(3686400L / baud);			// Baud Rate Cul Source = Clock(3.6864MHz) / baud
	UBR00 = (unsigned char)(datau & 0xff);			// Baud Rate Register Low
	UBR10 = (unsigned char)((datau >> 8) & 0xff);		// Baud Rate Register High
	UMCTL0 = 0x00;						// Modulation Control Register
	ME1 |= UTXE0 + URXE0;					// Enabled USART0 TXD/RXD
	IE1 |= URXIE0;						// Enabled USART0 RX interrupt
	
	P3SEL |= 0x30;						// P3.4,5 = USART0 TXD/RXD
	P3DIR |= 0x10;						// P3.4 output direction
}

// =============================== 1 Ʈ ۽ ==============================
// [μ] int c
// Item1:  (0 - ffh)
// [] printf()Լ  putchar() .
// ----------------------------------------------------------------------------
int putchar(int c)
{
	while(!(IFG1 & UTXIFG0));				// USART0 TX buffer ready?
	TXBUF0 = c;						// RXBUF0 = send data
	return(c);
}

// =============================== 1Ʈ ۽ ===============================
// [μ] unsigned char datac
// Item1:  (0 - ffh)
// ----------------------------------------------------------------------------
void putchar_0(unsigned char datac)
{
	while(!(IFG1 & UTXIFG0));				// USART0 TX buffer ready?
	TXBUF0 = datac;						// RXBUF0 = send data
}

// ================================ Ʈ ۽ ===============================
// [μ] unsigned char dbuf
// Item1:   ڿ
// ----------------------------------------------------------------------------
void puts_0(unsigned char dbuf[])
{
unsigned char ch;
int i = 0;

	ch = dbuf[i++];
	while(ch != 0){
		putchar_0(ch);					// 1 send
		ch = dbuf[i++];
	}
}

// ============================== UART0 interrupt =============================
// [μ] void
// ----------------------------------------------------------------------------
interrupt (UART0RX_VECTOR)usart0_rx(void)
{
	com0_rx_data = RXBUF0;					//   
	com0_rx_eflg = 1;					//  Ϸ ÷ SET
}
